/* hexdump.c:   dump files to standard output in hex/ASCII format */
/* 000626 DLO: mods:
/*   remove output titles, expand offset to five digits,
/*   allow input from stdin, allow output to named file
*/

#include <stdio.h>
#include <stdlib.h>

#define GUTTER 8        /* Print gutter after 8 items */

#define ERR_INFILE    1          /* Could not open input file */
#define ERR_READ      2          /* Could not read input file */
#define ERR_OUTFILE   3          /* Could not open output file */
#define ERR_WRITE     4          /* Could not write to output file */

int dump(fin, fout)
FILE *fin, *fout;
{
    unsigned char buf[16];
    int count, i;
    long size;
    
    size = 0;
    
    while ((count = fread(buf,1,16,fin)) > 0)
    {
        fprintf(fout, "%05X ",size);

        size+=count;
        /* Print Hex Bytes - Save text bytes */
        for (i = 0; i < 16; ++i)
        {
            if (i < count)
            {
                fprintf(fout, " %02X", buf[i]);
                if (buf[i]<0x20 || buf[i]>0x7f)
                    buf[i] = '.';
            }
            else
            {
                fputs("   ",fout);
                buf[i] = ' ';
            }
            if (i+1 == GUTTER)
                fputs("   ",fout);
        }
       
        /* Print Text Bytes */
        fprintf(fout, " |%16.16s|\n", buf);
    }
    return 0;
}

main(argc,argv)
int argc;
char *argv[];
{
    FILE *fin, *fout;
    int i, j;

    fin = stdin;
    fout = stdout;
    i = 1;
    j = 0;
    while(i < argc)
    {   j++;
        switch(j)
        {   case 1:                /* input file */
                fin=fopen(argv[i], "rb");  /* otherwise, open given file */
                if(!fin)
                {   fprintf(stderr, "Cannot open %s for input\n", argv[1]);
                    exit(ERR_INFILE);
                }
                break;

            case 2:                 /* output file */
                fout=fopen(argv[i], "wt");
                if(!fout)
                {   fprintf(stderr, "Cannot open %s for output\n", argv[2]);
                    if(fin != stdin)
                        fclose(fin);
                    exit(ERR_OUTFILE);
                }
        }
        i++;
    }

    dump(fin, fout);
    if(fin != stdin)
        fclose(fin);
    if(fout != stdout)
        fclose(fout);
    return 0;
}
