/****************************************************************************
 * undump.c
 *
 * Description:
 *   dump.exe converts a binary file to ASCII.  undump will take the
 *   output of dump and reconstruct the original binary file.
 *
 * Author: DL Ormand
 * Version: 1.0
 * Modification Record:
 *   060830   Initial Release                            DL Ormand
 ***************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* Local Prototypes */
void closefiles(FILE *fin, FILE *fout);

/****************************************************************************
 * main()
 *
 * Main routine.  In this case, that's pretty much all there is!
 *
 * INPUTS:
 *   int argc, number of command line arguments
 *   char **argv, array of pointers to command line arguments
 *
 * OUTPUTS:
 *   Zero if success, 1 if output file error
 ***************************************************************************/
int main(int argc, char **argv)
{
    FILE *fin, *fout;
    int i, j;
    char buf[100];
    char outbuf[20];
    char *p;

    /* Default to using stdin, stdout */
    fin = stdin;
    fout = stdout;
    
    /* If more than one command line argument (command itself), the first
     * argument is the source file.  Open as text */
    if(argc > 1) {
        if((fin = fopen(argv[1], "rt")) == NULL) {
            fprintf(stderr, "undump: cannot open %s for input\n", argv[1]);
            exit(0);
        }
    }

    /* If more than two command line arguments, the second argument is
     * the destination binary file.  Open as binary.  Note that under
     * DOS, this is the only way to get the original file back; writing
     * text files (including writing to stdout) replaces carriage returns
     * (>0D) with CR/LF pairs (>0D >0A)) */
    if(argc > 2) {
        if((fout = fopen(argv[2], "wb")) == NULL) {
            fprintf(stderr, "undump: cannot open %s for output\n", argv[2]);
            fclose(fin);
            exit(0);
        }
    }

    /* Process all the lines in the source file */
    while(fgets(buf, 100, fin) != NULL) {
        /* Find the first item in the line, which is the file offset */
        p = strtok(buf, " ");
        /* If found (otherwise end of file), convert each subsequent item
         * (bytes) from hex to binary, load a holding buffer, count them
         * (accommodate partial lines at the end), then write the buffer
         * to the output */
        if(p != NULL) {
            for(i=0; *(p = strtok(NULL, " ")) != '|'; i++) {
                sscanf(p, "%x", &j);
                outbuf[i] = (char)j;
            }
            if(i != fwrite(outbuf, 1, i, fout)) {
                fprintf(stderr, "undump: output file error\n");
                closefiles(fin, fout);
                exit(0);
            }
        }
    }

    closefiles(fin, fout);

    return 0;
}

/****************************************************************************
 * closefiles()
 *
 * Utility function to close the input and output files unless they are
 * the stdin, stdout defaults.
 *
 * INPUTS:
 *   FILE *fin, input file pointer (default is stdin)
 *   FILE *fout, output file pointer (default is stdout)
 *
 * OUTPUT: None
 ***************************************************************************/
void closefiles(FILE *fin, FILE *fout)
{
    if(fin != stdin) {
        fclose(fin);
    }
    if(fout != stdout) {
        fclose(fout);
    }
}

